@@ -66,6 +66,9 @@
 #define SSH_COM_AGENT2_FAILURE                   102
 
 #define SSH_AGENT_OLD_SIGNATURE                  0x01
+/* Signature flags from draft-miller-ssh-agent-02 */
+#define SSH_AGENT_RSA_SHA2_256                   0x02
+#define SSH_AGENT_RSA_SHA2_512                   0x04
 
 struct ssh_agent_struct {
   struct ssh_socket_struct *sock;
@@ -101,7 +104,7 @@ void ssh_agent_free(struct ssh_agent_struct *agent);
  */
 int ssh_agent_is_running(struct ssh_session_struct *session);
 
-int ssh_agent_get_ident_count(struct ssh_session_struct *session);
+uint32_t ssh_agent_get_ident_count(struct ssh_session_struct *session);
 
 ssh_key ssh_agent_get_next_ident(struct ssh_session_struct *session,
                                  char **comment);
