@@ -22,6 +22,7 @@
 #define ECDH_H_
 
 #include "config.h"
+#include "libssh/callbacks.h"
 
 #ifdef HAVE_LIBCRYPTO
 #ifdef HAVE_OPENSSL_ECDH_H
@@ -33,11 +34,24 @@
 #endif /* HAVE_OPENSSL_ECDH_H */
 #endif /* HAVE_LIBCRYPTO */
 
+#ifdef HAVE_GCRYPT_ECC
+#define HAVE_ECDH 1
+#endif
+
+#ifdef HAVE_LIBMBEDCRYPTO
+#define HAVE_ECDH 1
+#endif
+
+extern struct ssh_packet_callbacks_struct ssh_ecdh_client_callbacks;
+/* Backend-specific functions.  */
 int ssh_client_ecdh_init(ssh_session session);
-int ssh_client_ecdh_reply(ssh_session session, ssh_buffer packet);
+void ssh_client_ecdh_remove_callbacks(ssh_session session);
+int ecdh_build_k(ssh_session session);
 
 #ifdef WITH_SERVER
-int ssh_server_ecdh_init(ssh_session session, ssh_buffer packet);
+extern struct ssh_packet_callbacks_struct ssh_ecdh_server_callbacks;
+void ssh_server_ecdh_init(ssh_session session);
+SSH_PACKET_CALLBACK(ssh_packet_server_ecdh_init);
 #endif /* WITH_SERVER */
 
 #endif /* ECDH_H_ */
