@@ -32,6 +32,7 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdbool.h>
 
 #if !defined(HAVE_STRTOULL)
 # if defined(HAVE___STRTOULL)
@@ -423,4 +424,6 @@ void explicit_bzero(void *s, size_t n);
 
 void ssh_agent_state_free(void *data);
 
+bool is_ssh_initialized(void);
+
 #endif /* _LIBSSH_PRIV_H */
