@@ -103,6 +103,21 @@ void _ssh_set_error_invalid(void *error, const char *function)
                    "Invalid argument in %s", function);
 }
 
+/**
+ * @internal
+ *
+ * @brief Reset the error code and message
+ *
+ * @param  error       The place to reset the error.
+ */
+void ssh_reset_error(void *error)
+{
+    struct ssh_common_struct *err = error;
+
+    ZERO_STRUCT(err->error.error_buffer);
+    err->error.error_code = 0;
+}
+
 /**
  * @brief Retrieve the error text message from the last error.
  *
