@@ -45,6 +45,16 @@ static int sshd_teardown(void **state) {
     return 0;
 }
 
+static int sshd_setup_hmac(void **state)
+{
+    torture_setup_sshd_server(state, false);
+    /* Set MAC to be something other than what the client will offer */
+    torture_update_sshd_config(state, "MACs hmac-sha2-512");
+
+    return 0;
+}
+
+
 static int session_setup(void **state) {
     struct torture_state *s = *state;
     int verbosity = torture_libssh_verbosity();
@@ -412,6 +422,20 @@ static void torture_algorithms_aes256_gcm(void **state)
     test_algorithm(s->ssh.session, NULL/*kex*/, "aes256-gcm@openssh.com", NULL);
 }
 
+static void torture_algorithms_aes128_gcm_mac(void **state)
+{
+    struct torture_state *s = *state;
+
+    test_algorithm(s->ssh.session, NULL/*kex*/, "aes128-gcm@openssh.com", "hmac-sha1");
+}
+
+static void torture_algorithms_aes256_gcm_mac(void **state)
+{
+    struct torture_state *s = *state;
+
+    test_algorithm(s->ssh.session, NULL/*kex*/, "aes256-gcm@openssh.com", "hmac-sha1");
+}
+
 static void torture_algorithms_3des_cbc_hmac_sha1(void **state) {
     struct torture_state *s = *state;
 
@@ -472,8 +496,7 @@ static void torture_algorithms_3des_cbc_hmac_sha2_512_etm(void **state) {
     test_algorithm(s->ssh.session, NULL/*kex*/, "3des-cbc", "hmac-sha2-512-etm@openssh.com");
 }
 
-#ifdef WITH_BLOWFISH_CIPHER
-#if ((OPENSSH_VERSION_MAJOR == 7 && OPENSSH_VERSION_MINOR < 6) || OPENSSH_VERSION_MAJOR <= 6)
+#if defined(WITH_BLOWFISH_CIPHER) && defined(OPENSSH_BLOWFISH_CBC)
 static void torture_algorithms_blowfish_cbc_hmac_sha1(void **state) {
     struct torture_state *s = *state;
 
@@ -533,9 +556,9 @@ static void torture_algorithms_blowfish_cbc_hmac_sha2_512_etm(void **state) {
 
     test_algorithm(s->ssh.session, NULL/*kex*/, "blowfish-cbc", "hmac-sha2-512-etm@openssh.com");
 }
-#endif
 #endif /* WITH_BLOWFISH_CIPHER */
 
+#ifdef OPENSSH_CHACHA20_POLY1305_OPENSSH_COM
 static void torture_algorithms_chacha20_poly1305(void **state)
 {
     struct torture_state *s = *state;
@@ -549,6 +572,20 @@ static void torture_algorithms_chacha20_poly1305(void **state)
                    "chacha20-poly1305@openssh.com",
                    NULL);
 }
+static void torture_algorithms_chacha20_poly1305_mac(void **state)
+{
+    struct torture_state *s = *state;
+
+    if (ssh_fips_mode()) {
+        skip();
+    }
+
+    test_algorithm(s->ssh.session,
+                   NULL, /*kex*/
+                   "chacha20-poly1305@openssh.com",
+                   "hmac-sha1"); /* different from the server */
+}
+#endif /* OPENSSH_CHACHA20_POLY1305_OPENSSH_COM */
 
 static void torture_algorithms_zlib(void **state) {
     struct torture_state *s = *state;
@@ -647,7 +684,7 @@ static void torture_algorithms_ecdh_sha2_nistp521(void **state) {
 }
 #endif
 
-#if ((OPENSSH_VERSION_MAJOR == 7 && OPENSSH_VERSION_MINOR >= 3) || OPENSSH_VERSION_MAJOR > 7)
+#ifdef OPENSSH_CURVE25519_SHA256
 static void torture_algorithms_ecdh_curve25519_sha256(void **state) {
     struct torture_state *s = *state;
 
@@ -657,9 +694,9 @@ static void torture_algorithms_ecdh_curve25519_sha256(void **state) {
 
     test_algorithm(s->ssh.session, "curve25519-sha256", NULL/*cipher*/, NULL/*hmac*/);
 }
-#endif
+#endif /* OPENSSH_CURVE25519_SHA256 */
 
-#if ((OPENSSH_VERSION_MAJOR == 6 && OPENSSH_VERSION_MINOR >= 5) || OPENSSH_VERSION_MAJOR > 6)
+#ifdef OPENSSH_CURVE25519_SHA256_LIBSSH_ORG
 static void torture_algorithms_ecdh_curve25519_sha256_libssh_org(void **state) {
     struct torture_state *s = *state;
 
@@ -669,7 +706,7 @@ static void torture_algorithms_ecdh_curve25519_sha256_libssh_org(void **state) {
 
     test_algorithm(s->ssh.session, "curve25519-sha256@libssh.org", NULL/*cipher*/, NULL/*hmac*/);
 }
-#endif
+#endif /* OPENSSH_CURVE25519_SHA256_LIBSSH_ORG */
 
 static void torture_algorithms_dh_group1(void **state) {
     struct torture_state *s = *state;
@@ -691,6 +728,16 @@ static void torture_algorithms_dh_group14(void **state) {
     test_algorithm(s->ssh.session, "diffie-hellman-group14-sha1", NULL/*cipher*/, NULL/*hmac*/);
 }
 
+static void torture_algorithms_dh_group14_sha256(void **state) {
+    struct torture_state *s = *state;
+
+    if (ssh_fips_mode()) {
+        skip();
+    }
+
+    test_algorithm(s->ssh.session, "diffie-hellman-group14-sha256", NULL/*cipher*/, NULL/*hmac*/);
+}
+
 static void torture_algorithms_dh_group16(void **state) {
     struct torture_state *s = *state;
 
@@ -864,8 +911,7 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_algorithms_3des_cbc_hmac_sha2_512_etm,
                                         session_setup,
                                         session_teardown),
-#ifdef WITH_BLOWFISH_CIPHER
-#if ((OPENSSH_VERSION_MAJOR == 7 && OPENSSH_VERSION_MINOR < 6) || OPENSSH_VERSION_MAJOR <= 6)
+#if defined(WITH_BLOWFISH_CIPHER) && defined(OPENSSH_BLOWFISH_CBC)
         cmocka_unit_test_setup_teardown(torture_algorithms_blowfish_cbc_hmac_sha1,
                                         session_setup,
                                         session_teardown),
@@ -884,11 +930,12 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_algorithms_blowfish_cbc_hmac_sha2_512_etm,
                                         session_setup,
                                         session_teardown),
-#endif
 #endif /* WITH_BLOWFISH_CIPHER */
+#ifdef OPENSSH_CHACHA20_POLY1305_OPENSSH_COM
         cmocka_unit_test_setup_teardown(torture_algorithms_chacha20_poly1305,
                                         session_setup,
                                         session_teardown),
+#endif /* OPENSSH_CHACHA20_POLY1305_OPENSSH_COM */
         cmocka_unit_test_setup_teardown(torture_algorithms_zlib,
                                         session_setup,
                                         session_teardown),
@@ -901,6 +948,9 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_algorithms_dh_group14,
                                         session_setup,
                                         session_teardown),
+        cmocka_unit_test_setup_teardown(torture_algorithms_dh_group14_sha256,
+                                        session_setup,
+                                        session_teardown),
         cmocka_unit_test_setup_teardown(torture_algorithms_dh_group16,
                                         session_setup,
                                         session_teardown),
@@ -915,16 +965,16 @@ int torture_run_tests(void) {
                                         session_setup,
                                         session_teardown),
 #endif /* WITH_GEX */
-#if ((OPENSSH_VERSION_MAJOR == 7 && OPENSSH_VERSION_MINOR >= 3) || OPENSSH_VERSION_MAJOR > 7)
+#ifdef OPENSSH_CURVE25519_SHA256
         cmocka_unit_test_setup_teardown(torture_algorithms_ecdh_curve25519_sha256,
                                         session_setup,
                                         session_teardown),
-#endif
-#if ((OPENSSH_VERSION_MAJOR == 6 && OPENSSH_VERSION_MINOR >= 5) || OPENSSH_VERSION_MAJOR > 6)
+#endif /* OPENSSH_CURVE25519_SHA256 */
+#ifdef OPENSSH_CURVE25519_SHA256_LIBSSH_ORG
         cmocka_unit_test_setup_teardown(torture_algorithms_ecdh_curve25519_sha256_libssh_org,
                                         session_setup,
                                         session_teardown),
-#endif
+#endif /* OPENSSH_CURVE25519_SHA256_LIBSSH_ORG */
 #if defined(HAVE_ECC)
         cmocka_unit_test_setup_teardown(torture_algorithms_ecdh_sha2_nistp256,
                                         session_setup,
@@ -938,10 +988,30 @@ int torture_run_tests(void) {
 #endif
     };
 
+    struct CMUnitTest tests_hmac[] = {
+        cmocka_unit_test_setup_teardown(torture_algorithms_aes128_gcm_mac,
+                                        session_setup,
+                                        session_teardown),
+        cmocka_unit_test_setup_teardown(torture_algorithms_aes256_gcm_mac,
+                                        session_setup,
+                                        session_teardown),
+#ifdef OPENSSH_CHACHA20_POLY1305_OPENSSH_COM
+        cmocka_unit_test_setup_teardown(torture_algorithms_chacha20_poly1305_mac,
+                                        session_setup,
+                                        session_teardown),
+#endif /* OPENSSH_CHACHA20_POLY1305_OPENSSH_COM */
+    };
+
     ssh_init();
 
     torture_filter_tests(tests);
     rc = cmocka_run_group_tests(tests, sshd_setup, sshd_teardown);
+    if (rc != 0) {
+        return rc;
+    }
+
+    torture_filter_tests(tests);
+    rc = cmocka_run_group_tests(tests_hmac, sshd_setup_hmac, sshd_teardown);
 
     ssh_finalize();
 
