@@ -571,23 +571,25 @@ static void *thread_pki_rsa_generate_key(void *threadid)
     session = ssh_new();
     assert_non_null(session);
 
-    rc = ssh_pki_generate(SSH_KEYTYPE_RSA, 1024, &key);
-    assert_ssh_return_code(session, rc);
-    assert_non_null(key);
+    if (!ssh_fips_mode()) {
+        rc = ssh_pki_generate(SSH_KEYTYPE_RSA, 1024, &key);
+        assert_ssh_return_code(session, rc);
+        assert_non_null(key);
 
-    rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
-    assert_int_equal(rc, SSH_OK);
-    assert_non_null(pubkey);
+        rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
+        assert_int_equal(rc, SSH_OK);
+        assert_non_null(pubkey);
 
-    sign = pki_do_sign(key, RSA_HASH, 20, SSH_DIGEST_SHA256);
-    assert_non_null(sign);
+        sign = pki_do_sign(key, RSA_HASH, 20, SSH_DIGEST_SHA256);
+        assert_non_null(sign);
 
-    rc = pki_signature_verify(session, sign, pubkey, RSA_HASH, 20);
-    assert_ssh_return_code(session, rc);
+        rc = ssh_pki_signature_verify(session, sign, pubkey, RSA_HASH, 20);
+        assert_ssh_return_code(session, rc);
 
-    ssh_signature_free(sign);
-    SSH_KEY_FREE(key);
-    SSH_KEY_FREE(pubkey);
+        ssh_signature_free(sign);
+        SSH_KEY_FREE(key);
+        SSH_KEY_FREE(pubkey);
+    }
 
     rc = ssh_pki_generate(SSH_KEYTYPE_RSA, 2048, &key);
     assert_ssh_return_code(session, rc);
@@ -600,7 +602,7 @@ static void *thread_pki_rsa_generate_key(void *threadid)
     sign = pki_do_sign(key, RSA_HASH, 20, SSH_DIGEST_SHA256);
     assert_non_null(sign);
 
-    rc = pki_signature_verify(session, sign, pubkey, RSA_HASH, 20);
+    rc = ssh_pki_signature_verify(session, sign, pubkey, RSA_HASH, 20);
     assert_ssh_return_code(session, rc);
 
     ssh_signature_free(sign);
@@ -618,7 +620,7 @@ static void *thread_pki_rsa_generate_key(void *threadid)
     sign = pki_do_sign(key, RSA_HASH, 20, SSH_DIGEST_SHA256);
     assert_non_null(sign);
 
-    rc = pki_signature_verify(session, sign, pubkey, RSA_HASH, 20);
+    rc = ssh_pki_signature_verify(session, sign, pubkey, RSA_HASH, 20);
     assert_true(rc == SSH_OK);
 
     ssh_signature_free(sign);
